#!/usr/bin/env bash

####
# VirtualBox headless VM start script, copyright (c) Markus Duft
# .. do with this what you want (public domain), no warranty ... :)
####

zenity=$(type -p zenity)
notify=$(type -p notify-send)

[[ -z ${zenity} || -z ${notify} ]] && \
    { 
        echo "zenity and notify-send are required for this script!"
        exit 1
    }

function error() { echo "$@"; exit 1; }

# gather together all the vms.
declare -a vms
declare -a running
declare -a vrdes

function update_all_vms() {
    vms=()
    eval $(VBoxManage list vms | sed -e 's,"\([^"]*\)" {\([^}]*\)},vms[${#vms[@]}]="\2|\1",g')
}

function update_running_vms() {
    running=()
    eval $(VBoxManage list runningvms | sed -e 's,"\([^"]*\)" {\([^}]*\)},running[${#running[@]}]="\2",g')
}

function update_vrde() {
    vrdes=()
    update_all_vms
    update_running_vms

    for vm in "${running[@]}"; do
        eval $(VBoxManage showvminfo ${vm%|*} --machinereadable | grep -E '(\<vrde\>|\<vrdeport\>)')

        if [[ ${vrdeport} == "-1" ]]; then
            zenity --warning --text "The virtual machine ${vm%|*} has an invalid VRDE port. This is most probably a conflict with another machine."
            continue
        fi

        if [[ ${vrde} == on ]]; then
            vrdes[${#vrdes[@]}]="${vrdeport}|${vm#*|}"
        fi
    done
}

function name_from_uuid() {
    for vm in "${vms[@]}"; do
        if [[ ${vm%|*} == $1 ]]; then
            echo "${vm#*|}"
            return
        fi
    done
}

function run_manager() {
    update_all_vms
    update_running_vms

    local cmd="zenity --list --title \"Choose VM\" --text \"Choose a Virtual Machine you want to start or stop\" --width=350 --height=250 --hide-column=1 --column \"UUID\" --column \"Name\" --column \"State\" "

    for vm in "${vms[@]}"; do
        local state=stopped
        [[ ${running[@]} == *${vm%|*}* ]] && state=started

        cmd="${cmd} \"${vm%|*}\" \"${vm#*|}\" \"${state}\""
    done

    local response=$(eval "${cmd}")

    if [[ -n ${response} ]]; then
        if [[ ${running[@]} == *${response}* ]]; then
            VBoxManage controlvm "${response}" savestate | zenity --progress --pulsate --auto-close --no-cancel --text="Saving state"
            if [[ $? == 0 ]]; then
                notify-send -t 4000 -i info "Virtual Machine state saved" "Virtual Machine $(name_from_uuid "${response}") was saved successfully"
            else
                notify-send -t 4000 -i error "Virtual Machine Error" "Virtual Machine $(name_from_uuid "${response}") could not be stopped!"
            fi
        else
            VBoxManage startvm "${response}" --type headless | zenity --progress --pulsate --auto-close --no-cancel --text="Starting or restoring"
            if [[ $? == 0 ]]; then
                notify-send -t 4000 -i info "Virtual Machine starting" "Virtual Machine $(name_from_uuid "${response}") is starting ..."
            else
                notify-send -t 4000 -i error "Virtual Machine Error" "Virtual Machine $(name_from_uuid "${response}") could not be started!"
            fi
        fi
    fi
}

function run_vrde() {
    update_vrde

    local cmd="zenity --list --title \"Choose VM\" --text \"Choose a Virtual Machine you want to connect to\" --width=350 --height=250 --hide-column=1 --column \"Port\" --column \"Name\" "

    for vm in "${vrdes[@]}"; do
        cmd="${cmd} \"${vm%|*}\" \"$(name_from_uuid "${vm#*|}")\""
    done

    echo $cmd

    local response=$(eval ${cmd})

    if [[ -n ${response} ]]; then
        for vm in "${vrdes[@]}"; do
            if [[ ${response} == ${vm%|*} ]]; then
                break
            fi

            vm=
        done

        [[ -n ${vm} ]] && vm=$(name_from_uuid "${vm#*|}")

        rdesktop -b localhost:${response} -T "VM: ${vm} (${response})" &
        local _id=$!

        sleep 1
        local ok=$(ps -ef | grep "\<${_id}\>" | grep -v '\<grep\>' | wc -l)

        if [[ ${ok} != 1 ]]; then
            notify-send -t 4000 -i error "Remote Desktop Error" "Could not connect to Virtual Machine on port ${response}..."
        fi
    fi
}

case "$1" in
manager) run_manager || error "error while running machine manager" ;;
vrdemanager) run_vrde || error "error while running vrde manager" ;;
*) error "invalid mode. pass 'manager' or 'vrdemanager'" ;;
esac
